/**************************************************************** 
 * Description: 洛谷：P3379  
 一个用于解决多叉树最近公共祖先（LCA, Lowest Common Ancestor）问题的C++程序。
 程序首先读入一棵多叉树和一系列查询，然后对于每个查询输出两个节点的最近公共祖先。
 这里使用了深度优先搜索（DFS）和倍增方法（Binary Lifting）来高效地解决这个问题。
 * Author: Alex Li
 * Date: 2024-01-24 15:04:53
 * LastEditTime: 2024-01-28 22:47:08
****************************************************************/
#include <iostream>
#include <cstdio>
#include <cstring>
#include <algorithm>
using namespace std;
struct zzz {
    int t, nex;
}e[1000010]; // 定义边的结构体，存储目标节点和下一个边的索引
int head[500010], tot; // head数组存储每个节点的第一条边的索引，tot用于计数
// depth存储节点深度，fa存储节点的祖先，lg用于优化查找
int depth[500001], fa[500001][22], lg[500001];

//// 添加边的函数
void add(int x, int y) {
	e[++tot].t = y;
	e[tot].nex = head[x];
	head[x] = tot;
}

// 深度优先搜索函数，用于构建树和初始化fa和depth数组
void dfs(int now, int fath) {
	fa[now][0] = fath; depth[now] = depth[fath] + 1;
	for(int i = 1; i <= lg[depth[now]]; ++i)
		fa[now][i] = fa[fa[now][i-1]][i-1];
	for(int i = head[now]; i; i = e[i].nex)
		if(e[i].t != fath) dfs(e[i].t, now);
}

// LCA函数，返回两个节点的最近公共祖先
//
int LCA(int x, int y) {
	if(depth[x] < depth[y]) swap(x, y);
	//将两个节点提升到同一深度。首先，通过while循环减少x的深度，使得x和y处于同一深度.
	while(depth[x] > depth[y])
		x = fa[x][lg[depth[x]-depth[y]] - 1];
	if(x == y) return x;
	
	//使用一个for循环向上跳跃，每次尽可能大的跳跃（即2^k跳跃），直到找到x和y的最近公共祖先。
	for(int k = lg[depth[x]] - 1; k >= 0; --k)
		if(fa[x][k] != fa[y][k])
			x = fa[x][k], y = fa[y][k];
	return fa[x][0];
}

int main() {
    ios::sync_with_stdio(false);
	int n, m, s; //n是节点数，m是查询数，s是根节点
// 	scanf("%d%d%d", &n, &m, &s);
	cin>>n>>m>>s;
	
	// 读入树的边
	for(int i = 1; i <= n-1; ++i) {
		int x, y; 
// 		scanf("%d%d", &x, &y);
		cin>>x>>y;
		add(x, y); add(y, x);
	}

	// 初始化lg数组，用于加速后面的查询
	for(int i = 1; i <= n; ++i)
		lg[i] = lg[i-1] + (1 << lg[i-1] == i);

	 // 构建树	
	dfs(s, 0);

	 // 对于每个查询，输出LCA
	for(int i = 1; i <= m; ++i) {
		int x, y; 
// 		scanf("%d%d",&x, &y);
		cin>>x>>y;
		//printf("%d\n", LCA(x, y));
		cout<<LCA(x,y)<<'\n';
	}
	return 0;
}